import React, { useEffect, useMemo, useState } from "react";
import { styled } from "@mui/material/styles";
import {
  Grid,
  Typography,
  Card,
  CardContent,
  Container,
  Link as MuiLink
} from "@mui/material";
import { DeploymentStepper } from "./DeploymentStepper";
import { KeyValueTable } from "@ess-ics/ce-ui-common/dist/components/common/KeyValueTable";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { DeploymentBadge } from "./DeploymentBadge";
import { DeploymentJobOutput } from "./DeploymentJobOutput";
import { Link as ReactRouterLink } from "react-router-dom";
import { formatDate } from "../common/Helper";
import GitRefLink from "../IOC/GitRefLink";
import AccessControl from "../auth/AccessControl";
import { AWXJobDetails, DeploymentStatus } from "../../api/DataTypes";
import AlertMessages from "../IOC/AlertMessages";

const PREFIX = "DeploymentDetails";

const classes = {
  csentryIdInvalid: `${PREFIX}-csentryIdInvalid`
};

const StyledGrid = styled(Grid)(({ theme }) => ({
  [`& .${classes.csentryIdInvalid}`]: {
    color: theme.csentryIdInvalid.color
  }
}));

function createAlert(deployment, deploymentJob) {
  const deploymentStatus = new DeploymentStatus(deployment, deploymentJob);
  const message = deploymentStatus.message();
  const severity = deploymentStatus.severity();
  return {
    type: severity,
    message: message
  };
}

export function DeploymentDetails({ deployment, deploymentJob }) {
  let deplAlert = deployment.alerts;

  const finishedJob = useMemo(
    () =>
      deployment &&
      new DeploymentStatus(deployment, deploymentJob).isFinished(),
    [deployment, deploymentJob]
  );

  const [alert, setAlert] = useState(
    createAlert(deployment, deploymentJob, finishedJob)
  );

  const jobDetails = useMemo(
    () =>
      new AWXJobDetails(
        deployment.undeployment ? "UNDEPLOY" : "DEPLOY",
        deploymentJob
      ),
    [deployment, deploymentJob]
  );

  useEffect(() => {
    setAlert(createAlert(deployment, deploymentJob));
  }, [deployment, deploymentJob]);

  function calculateHostText() {
    // host is resolvable => show link for users
    if (deployment.host.csEntryId && deployment.host.csEntryIdValid) {
      return (
        <Typography>
          <MuiLink
            component={ReactRouterLink}
            to={`/hosts/${deployment.host.csEntryId}`}
            underline="hover"
          >
            {deployment.host.fqdn}
          </MuiLink>
        </Typography>
      );
    }

    // gray out hostname if it is from cache
    return (
      <Typography className={classes.csentryIdInvalid}>
        {deployment.host.fqdn}
      </Typography>
    );
  }

  const deploymentDetails = {
    Revision: (
      <GitRefLink
        url={deployment?.sourceUrl}
        revision={deployment?.sourceVersion}
      />
    ),
    host: calculateHostText(),
    user: deployment.createdBy,
    repository: (
      <Typography>
        <MuiLink
          href={deployment.sourceUrl}
          target="_blank"
          rel="noreferrer"
          underline="hover"
        >
          {deployment.sourceUrl}
        </MuiLink>
      </Typography>
    ),
    "AWX job link": (
      <Typography>
        <MuiLink
          href={deployment.awxJobUrl}
          target="_blank"
          rel="noreferrer"
          underline="hover"
        >
          {deployment.awxJobUrl}
        </MuiLink>
      </Typography>
    ),
    "created time": formatDate(deployment?.createdAt),
    "awx job start time": deploymentJob?.started
      ? formatDate(deploymentJob.started)
      : "-"
  };

  return (
    <StyledGrid
      container
      spacing={1}
    >
      {finishedJob && (
        <Grid
          item
          xs={12}
        >
          <AlertMessages alerts={[alert].concat(deplAlert)} />
        </Grid>
      )}
      <Grid
        item
        xs={12}
        md={12}
      >
        <SimpleAccordion
          defaultExpanded
          summary={<DeploymentBadge deployment={deployment} />}
        >
          <KeyValueTable
            obj={deploymentDetails}
            variant="overline"
            sx={{ border: 0 }}
            valueOptions={{ headerName: "" }}
          />
        </SimpleAccordion>
      </Grid>
      <Grid
        item
        xs={12}
        md={12}
      >
        <Card>
          <CardContent>
            <div>
              {deploymentJob && (
                <DeploymentStepper
                  activeStep={["queued", "running", "successful"].indexOf(
                    deploymentJob.status.toLowerCase()
                  )}
                  deploymentStart={deployment.startDate}
                  jobDetails={jobDetails}
                />
              )}
            </div>
          </CardContent>
        </Card>
      </Grid>
      {!finishedJob && alert && (
        <Grid
          item
          xs={12}
        >
          <AlertMessages alerts={[alert]} />
        </Grid>
      )}
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        <Grid
          item
          xs={12}
          md={12}
          style={{ paddingBottom: 0 }}
        >
          {deploymentJob ? (
            <SimpleAccordion
              summary="Ansible Job Output"
              defaultExpanded
            >
              <Container>
                <DeploymentJobOutput deploymentJob={deploymentJob} />
              </Container>
            </SimpleAccordion>
          ) : (
            <></>
          )}
        </Grid>
      </AccessControl>
    </StyledGrid>
  );
}
